/*---------------------------------------------------------------------------

    Filename: lokad_client.js
    Project:  Lokad API
    (C) Copyright 2008, Kirix Corporation, All Rights Reserved.

    Summary:         Grants the ability to interact with the Lokad Timeseries
                     web server and get forecasts for one or multiple series
    Developer(s):    Jeff Engel

---------------------------------------------------------------------------*/


// to run this script without using the gui, uncomment the following lines
// fill out the appropriate information, and select "Tools"->"Run Script/Query";
// this script can then be scheduled to run automatically using the job
// scheduler, found under "Tools"->"Schedule Jobs..."

/*

// --- create a new serie and add some data to it ---

// TODO: enter a name for the serie as it will be stored on the server
var serie = new LokadSerie("TestSerie");

// TODO: enter the timezone offset of the data
var today = new Date();
serie.setUtcOffset(-today.getTimezoneOffset() / 60);

// add data to the serie
// TODO: modify this as you wish
for(var i = 1; i <= 30; i++)
{
    var tsv = new LokadTimeValue;
    tsv.time = new Date(today.getFullYear(), 0, i, 0, 0, 0, 0);
    tsv.value = 10 * Math.log(i/2);
    serie.addTimeValue(tsv);
}


// --- create a forecasting task based on the serie ---

// TODO: enter a name for the task as it will be stored on the server
var task = new LokadTask("TestTask", serie);

// TODO: enter an aggregator;
//       valid aggregators include: Sum, Minimum, Maximum, First, Last, Average,
//                                  Count, StandardDeviation and Variance
task.setAggregator("Sum");

// TODO: enter the type of period
//       valid periods include: QuarterHour, HalfHour, Hour, Day, Week,
//                              Month, Quarter, Semester, and Year
task.setPeriod("Day");

// TODO: enter the number of previous records to include in the forecast
task.setPastPeriods(0);

// TODO: enter the desired forecast output, including the number of forecast
//       records to include and the number of past periods to include;
//       from the Lokad API documentation:
//              "Limitations: for performance reasons, Task.FuturePeriods is
//              upper-bounded; the actual upper-bound depends from Task.Period.
//              For Period.Hour, Period.HalfHour and Period.QuarterHour, the
//              limit is set to two weeks, e.g. 336, 672 and 1344 respectively.
//              For all all the other periods, the limit is set to 64."
task.setFuturePeriods(5);



// --- create the forecaster client that will connect to the server ---

// TODO: change this url if using the sandbox server
//var lokadClient = new LokadClient('http://ws.lokad.com/TimeSeries.asmx');

// TODO: enter your Lokad account username & password
//lokadClient.setAuthentication('username', 'password');


// --- upload the serie to the server and download the forecast ---
try
{
    if (!lokadClient.isAuthenticated())
    {
        alert("Sorry, could not log in to the server.\n"
            + "Please check your username and password and try again.");
    }
    else
    {
        // upload the serie to the server
        lokadClient.addSerie(serie);

        // create the forecasting task on the server
        lokadClient.addTask(task);

        // get the forecast from the server
        var forecast = lokadClient.getForecast(task);

        var timeValues = forecast.getTimeValues();
        var str = "Forecast for " + forecast.getName() + ":\n";
        for(var i in timeValues)
            str += "\n" + timeValues[i].time + ":\t" + timeValues[i].value;
        alert(str);

    }
}
catch (e)
{
    alert(e);
}

*/




include "lokad_serie.js";
include "lokad_task.js";

// (CLASS) LokadClient
// Category: Lokad Forecaster
// Derives: Object
// Description: A class for communicating with the Lokad web servers
// Remarks: The LokadClient class allows 2-way communication with the Lokad
//      web servers.
class LokadClient
{
    // (CONSTRUCTOR) LokadClient.constructor
    // Description: Creates a new LokadClient object.
    //
    // Syntax: LokadClient(url : String)
    //
    // Remarks: Creates a new LokadClient that communicates via the
    //      specified |url|. This enables use with both the sandbox servers and
    //      the production servers
    //
    // Param(url): The URL used to communicate with the Lokad servers.
    function LokadClient(url)
    {
        m_url = url;
    }

    // (METHOD) LokadClient.setAuthentication
    // Description: Sets the authentication credentials that are used when
    //      communicating with the server.
    //
    // Syntax: LokadSerie.setAuthentication(username: String
    //                                      password: String)
    //
    // Remarks: Sets the authentication credentials that are used when
    //      communicating with the server.
    //
    // Param(username): The username of the Lokad account to use.
    // Param(password): The password of the Lokad account to use.
    function setAuthentication(username, password)
    {
        m_username = username;
        m_password = password;
    }


    // --- functions that invoke an interaction with the Lokad server ---


    // (METHOD) LokadClient.addSerie
    // Description: Imports a time-serie with the specified values.
    //
    // Syntax: addSerie(serie: LokadSerie)
    //
    // Remarks: Stores a LokadSerie object onto the Lokad servers.  Note the
    //      time-serie may not include more than 25000 time-values.  If it is
    //      necessary to import more than 25000 time-values, then
    //      |LokadSerie.updateTimeValues| should be used instead.
    //
    // Param(serie): The |serie| to save on the servers.
    function addSerie(serie)
    {
        _invoke("AddSerie", serie,
                       _generateAddSerie, undefined);
    }

    // (METHOD) LokadClient.addSeries
    // Description: Imports an Array of time-serie with the specified values.
    //
    // Syntax: addSeries(series: Array(LokadSerie))
    //
    // Remarks: Stores multiple LokadSerie objects onto the Lokad servers.
    //
    // Param(serie): The Array of |series| to save on the servers.
    function addSeries(series)
    {
        // no more than 500 time-series may be added at once, so split large
        // requests into chunks of 500
        for(var chunk = 0; chunk < series.length; chunk += 500)
        {
            _invoke("AddSeries", series.slice(chunk, chunk + 500),
                           _generateAddSeries, undefined);
        }
    }

    // (METHOD) LokadClient.addTask
    // Description: Creates a new forecasting task associated to a specified
    //      time-serie.
    //
    // Syntax: addTask(task: LokadTask)
    //
    // Remarks: Creates a new forecasting task associated to a specified
    //      time-serie.
    //
    // Param(task): The |task| to save on the servers.
    function addTask(task)
    {
        _invoke("AddTask", task,
                       _generateAddTask, undefined);
    }

    // (METHOD) LokadClient.addTasks
    // Description: Creates new forecasting tasks associated to specified
    //      time-series.
    //
    // Syntax: addTasks(task: Array(LokadTask))
    //
    // Remarks: Creates new forecasting tasks associated to specified
    //      time-series.
    //
    // Param(tasks): The Array of |tasks| to save on the servers.
    function addTasks(tasks)
    {
        // no more than 500 tasks may be added at once, so split large
        // requests into chunks of 500
        for(var chunk = 0; chunk < tasks.length; chunk += 500)
        {
            _invoke("AddTasks", tasks.slice(chunk, chunk + 500),
                           _generateAddTasks, undefined);
        }
    }

    // (METHOD) LokadClient.deleteSerie
    // Description: Deletes a time-serie all the associated data.
    //
    // Syntax: deleteSerie(serie: LokadSerie)
    //
    // Remarks: Deletes a time-serie all the associated data (tasks, forecasts)
    //      from the Lokad servers.
    //
    // Param(serie): The |serie| to delete from the servers.
    function deleteSerie(serie)
    {
        _invoke("DeleteSerie", serie,
                       _generateDeleteSerie, undefined);
    }

    // (METHOD) LokadClient.deleteSeries
    // Description: Deletes a set of time-series with all associated data.
    //
    // Syntax: deleteSerie(series: Array(LokadSerie))
    //
    // Remarks: Deletes a set of time-series with all the associated data
    //      (tasks, forecasts) from the Lokad servers.
    //
    // Param(series): The Array of |series| to delete from the servers.
    function deleteSeries(series)
    {
        // no more than 500 time-series may be deleted at once, so split large
        // requests into chunks of 500
        for(var chunk = 0; chunk < series.length; chunk += 500)
        {
            _invoke("DeleteSeries", series.slice(chunk, chunk + 500),
                           _generateDeleteSeries, undefined);
        }
    }

    // (METHOD) LokadClient.deleteTask
    // Description: Deletes a task.
    //
    // Syntax: deleteTask(task: LokadTask)
    //
    // Remarks: Deletes a task from the Lokad servers.
    //
    // Param(task): The |task| to delete from the servers.
    function deleteTask(task)
    {
        _invoke("DeleteTask", task,
                       _generateDeleteTask, undefined);
    }

    // (METHOD) LokadClient.deleteTasks
    // Description: Deletes a set of tasks.
    //
    // Syntax: deleteTask(tasks: Array(LokadTask))
    //
    // Remarks: Deletes a set of tasks from the Lokad servers.
    //
    // Param(tasks): The Array of |tasks| to delete from the servers.
    function deleteTasks(tasks)
    {
        // no more than 500 tasks may be deleted at once, so split large
        // requests into chunks of 500
        for(var chunk = 0; chunk < tasks.length; chunk += 500)
        {
            _invoke("DeleteTasks", tasks.slice(chunk, chunk + 500),
                           _generateDeleteTasks, undefined);
        }
    }

    // (METHOD) LokadClient.getForecast
    // Description: Gets the forecast results associated to the specified
    //      forecasting task.
    //
    // Syntax: getForecast() : LokadForecast
    //
    // Remarks: Gets the forecast results associated to the specified
    //      forecasting task.
    //
    // Returns: A LokadForecast object containing the forecast time-serie data.
    function getForecast(task)
    {
        return _invoke("GetForecast", task,
                       _generateGetForecast, _parseGetForecast);
    }

    // (METHOD) LokadClient.isAuthenticated
    // Description: Indicates whether the client is authenticated or not.
    //
    // Syntax: isAuthenticated() : bool
    //
    // Remarks: Indicates whether the client is authenticated or not.
    //
    // Returns: A boolean value indicating whether the client could
    //      authenticate against the servers.
    function isAuthenticated()
    {
        return _invoke("IsAuthenticated", undefined,
                       undefined, _parseIsAuthenticated);
    }

    // (METHOD) LokadClient.setPartner
    // Description: Associates an authenticated call to a partner account.
    //
    // Syntax: setPartner(partnerId: Integer)
    //
    // Remarks: Associates an authenticated call to a partner account.
    //
    // Param(partnerId): The identifier of the partner account.
    function setPartner(partnerId)
    {
        _invoke("SetPartner", partnerId,
                       _generateSetPartner, undefined);
    }


    // --- private member functions ---

    // sends an action request to the Lokad server
    function _invoke(action, arg, generatorFxn, parseFxn)
    {
        // build the xml for the request
        var xml = _buildXML(action, arg, generatorFxn);
        if(xml == null)
            throw "Error building XML";

        var request = new HttpRequest;
        request.setUserAgent("Kirix Strata Lokad Client/0.0.0");
        request.setAutoEncode(false);
        request.setRequestHeader("Content-Type: text/xml");
        request.setRequestHeader("SoapAction: lokad.com/ws/" + action);
        request.setPostData(xml);

        // send the request to the server and get the results
        var returnVal = request.post(m_url);

        // parse the results of the request into an xml structure
        var rootNode = new XmlNode;
        rootNode.parse(returnVal);

        if(!rootNode || !rootNode.getChildNodes().length)
        {
            throw "Error retrieving server response.\n\n"
                    + "Please check your connection.";
        }

        var bodyNode = rootNode.getChildNodes()[0];
        var bodysNodes = bodyNode.getChildNodes();

        // look to see if a fault was thrown by the server
        if(bodysNodes.length && bodysNodes[0].getNodeName() == "soap:Fault")
        {
            throw "The server returned an error:\n\n"
                    + bodysNodes[0].getChildNodes()[1].getNodeValue();
        }

        // make sure the proper response was returned
        if(!bodysNodes.length ||
           bodysNodes[0].getNodeName() != action + "Response")
        {
            throw "Error parsing the response.\n\n"
                    + "What the server sent back was not expected.";
        }

        if(parseFxn != undefined)
        {
            this.thunk = parseFxn;
            return this.thunk(bodysNodes[0].getChildNodes()[0]);
        }
    }

    // builds the core xml structure for a request to the Lokad server;
    // if a generatorFxn is supplied, the output from that function
    // will be included in the xml
    function _buildXML(action, arg, generatorFxn)
    {
        var root = new XmlNode;
        root.setNodeName("soap:Envelope");
        root.appendAttribute("xmlns:soap",
                            "http://schemas.xmlsoap.org/soap/envelope/");


        var headers = root.appendChild("soap:Header");

        var auth = headers.appendChild("AuthHeader");
        auth.appendAttribute("xmlns", "lokad.com/ws");
        auth.appendChild("UserName", m_username);
        auth.appendChild("Password", m_password);


        var body = root.appendChild("soap:Body");

        var actionNode = body.appendChild(action);
        actionNode.appendAttribute("xmlns", "lokad.com/ws");
        if(generatorFxn != undefined)
        {
            this.thunk = generatorFxn;
            if(this.thunk(actionNode, arg) === false)
                return null;
        }


        return root.saveToString();
    }

    // pads a string with zeros so that the output is the specified length
    function _zeroPad(number, count)
    {
        var str = "00000000" + number;
        return str.substr(str.length - count);
    }
    
    // transforms a Date object into the required wsdl syntax
    function _timeToWsdl(time)
    {
        // TODO: remove the workarounds from this section
        //       once bug #1069 gets fixed
        return time.getFullYear() + "-" +
                _zeroPad(time.getMonth()+1, 2) + "-" +
                _zeroPad(time.getDate(), 2) + "T" +
                _zeroPad((time.getMinutes() == 60 ? 1 : 0)
                        + time.getHours(), 2) + ":" +
                _zeroPad(time.getMinutes() == 60 ? 0 :
                        time.getMinutes(), 2) + ":" +
                _zeroPad(time.getSeconds(), 2);
    }
    
    // transforms a wsdl time string into a Date object
    function _wsdlToTime(wsdlstr)
    {
        return new Date(wsdlstr.split("T").join(" "));
    }





    // Lokad SOAP function: AddSerie
    function _generateAddSerie(parentNode, arg)
    {
        var serieNode = parentNode.appendChild("serie");
        serieNode.appendAttribute("Name", arg.getName());
        if(arg.getUnitName() !== undefined)
            serieNode.appendAttribute("UnitName", arg.getUnitName());
        serieNode.appendAttribute("UtcOffset", arg.getUtcOffset());

        var tvaluesNode = serieNode.appendChild("TimeValues");

        var tvals = arg.getTimeValues();
        var numvals = 0;
        for(var i in tvals)
        {
            var tvnode = tvaluesNode.appendChild("TimeValue");
            tvnode.appendAttribute("Time", _timeToWsdl(tvals[i].time));
            tvnode.appendAttribute("Value", tvals[i].value);
            numvals++;
            if(numvals >= 25000)
                break;
        }
    }

    // Lokad SOAP function: AddSeries
    function _generateAddSeries(parentNode, arg)
    {
        var seriesNodes = parentNode.appendChild("series");

        for(var ii in arg)
        {
            var serieNode = seriesNodes.appendChild("TimeSerie");
            serieNode.appendAttribute("Name", arg[ii].getName());
            if(arg[ii].getUnitName() !== undefined)
                serieNode.appendAttribute("UnitName", arg[ii].getUnitName());
            serieNode.appendAttribute("UtcOffset", arg[ii].getUtcOffset());

            var tvaluesNode = serieNode.appendChild("TimeValues");

            var tvals = arg[ii].getTimeValues();
            var numvals = 0;
            for(var i in tvals)
            {
                var tvnode = tvaluesNode.appendChild("TimeValue");
                tvnode.appendAttribute("Time", _timeToWsdl(tvals[i].time));
                tvnode.appendAttribute("Value", tvals[i].value);
                numvals++;
                if(numvals >= 25000)
                    break;
            }
        }
    }

    // Lokad SOAP function: AddTask
    function _generateAddTask(parentNode, arg)
    {
        parentNode.appendChild("serieName", arg.getSerie().getName());

        var taskNode = parentNode.appendChild("task");
        taskNode.appendAttribute("Name", arg.getName());
        taskNode.appendAttribute("Aggregator", arg.getAggregator());
        taskNode.appendAttribute("Period", arg.getPeriod());
        if(arg.getPeriodStart() !== undefined)
            taskNode.appendAttribute("PeriodStart",
                        arg.getPeriodStart().getFullYear() + "-" +
                        arg.getPeriodStart().getMonth() + "-" +
                        arg.getPeriodStart().getDate());
        taskNode.appendAttribute("PastPeriods", arg.getPastPeriods());
        taskNode.appendAttribute("FuturePeriods", arg.getFuturePeriods());
    }

    // Lokad SOAP function: AddTasks
    function _generateAddTasks(parentNode, arg)
    {
        var seriesNode = parentNode.appendChild("serieNames");
        var tasksNode = parentNode.appendChild("tasks");

        for(var ii in arg)
        {
            seriesNode.appendChild("string", arg[ii].getSerie().getName());

            var taskNode = tasksNode.appendChild("Task");
            taskNode.appendAttribute("Name", arg[ii].getName());
            taskNode.appendAttribute("Aggregator", arg[ii].getAggregator());
            taskNode.appendAttribute("Period", arg[ii].getPeriod());
            if(arg[ii].getPeriodStart() !== undefined)
                taskNode.appendAttribute("PeriodStart",
                            arg[ii].getPeriodStart().getFullYear() + "-" +
                            arg[ii].getPeriodStart().getMonth() + "-" +
                            arg[ii].getPeriodStart().getDate());
            taskNode.appendAttribute("PastPeriods", arg[ii].getPastPeriods());
            taskNode.appendAttribute("FuturePeriods", arg[ii].getFuturePeriods());
        }
    }

    // Lokad SOAP function: DeleteSerie
    function _generateDeleteSerie(parentNode, arg)
    {
        parentNode.appendChild("serieName", arg.getName());
    }

    // Lokad SOAP function: DeleteSeries
    function _generateDeleteSeries(parentNode, arg)
    {
        var seriesNode = parentNode.appendChild("serieNames");
        
        for(var ii in arg)
            parentNode.appendChild("string", arg.getName());
    }

    // Lokad SOAP function: DeleteTask
    function _generateDeleteTask(parentNode, arg)
    {
        parentNode.appendChild("serieName", arg.getSerie().getName());
        parentNode.appendChild("taskName", arg.getName());
    }

    // Lokad SOAP function: DeleteTasks
    function _generateDeleteTasks(parentNode, arg)
    {
        var seriesNode = parentNode.appendChild("serieNames");
        var tasksNode = parentNode.appendChild("taskNames");

        for(var ii in arg)
        {
            seriesNode.appendChild("string", arg.getSerie().getName());
            tasksNode.appendChild("string", arg.getName());
        }
    }

    // Lokad SOAP function: GetForecast
    function _generateGetForecast(parentNode, arg)
    {
        parentNode.appendChild("serieName", arg.getSerie().getName());
        parentNode.appendChild("taskName", arg.getName());
    }
    function _parseGetForecast(resultNode)
    {
        var attrs = resultNode.getAttributes();

        var forecast = new LokadForecast(attrs.Name);
        forecast.setUnitName(attrs.UnitName);
        forecast.setUtcOffset(attrs.UtcOffset);

        var valueNodes = resultNode.getChildNodes()[0].getChildNodes();

        for(var i in valueNodes)
        {
            var args = valueNodes[i].getAttributes();
            var timeValue = new LokadTimeValue;
            timeValue.time = _wsdlToTime(args.Time);
            timeValue.value = args.Value;
            forecast.addTimeValue(timeValue);
        }

        return forecast;
    }

    // Lokad SOAP function: IsAuthenticated
    function _parseIsAuthenticated(resultNode)
    {
        return resultNode.getNodeValue() == "true";
    }

    // Lokad SOAP function: SetPartner
    function _generateSetPartner(parentNode, arg)
    {
        parentNode.appendChild("partnerId", arg);
    }



    // private member variables
    var m_db = new DbConnection;
    var m_url, m_username, m_password;
}
