/*---------------------------------------------------------------------------

    Filename: lokad_serie.js
    Project:  Lokad API
    (C) Copyright 2008, Kirix Corporation, All Rights Reserved.

    Summary:         A class for storing a set of time-serie values
    Developer(s):    Jeff Engel

---------------------------------------------------------------------------*/


// (CLASS) LokadSerie
// Category: Lokad Forecaster
// Derives: Object
// Description: A class for storing a set of time-serie values
// Remarks: LokadSerie is equivalent to a Serie that is stored on the Lokad
//      servers
class LokadSerie
{
    // (CONSTRUCTOR) LokadSerie.constructor
    // Description: Creates a new LokadSerie object.
    //
    // Syntax: LokadSerie(name : String)
    //
    // Remarks: Creates a new LokadSerie object with the identifier |name|.
    //
    // Param(name): The unique name of the LokadSerie as it will be stored and
    //      referenced on the Lokad servers. It should be distinct from any
    //      other time-serie names existing in your account and should be less
    //      than 64 characters. Additionally, the following list of characters
    //      ; [ ] < > \ / " ' are prohibited.
    function LokadSerie(name)
    {
        m_name = name.substr(0, 64);
        m_unitname = "";
        m_utcoffset = 0;
        m_timevalues = new Array;
    }

    // (METHOD) LokadSerie.getName
    // Description: Gets the identifier of this serie object.
    //
    // Syntax: LokadSerie.getName() : String
    //
    // Remarks: Returns the identifier of this serie object.
    //
    // Returns: The identifier of this serie object as a string.
    function getName()
    {
        return m_name;
    }

    // (METHOD) LokadSerie.getUnitName
    // Description: Gets the unit name of the serie values.
    //
    // Syntax: LokadSerie.getUnitName() : String
    //
    // Remarks: Returns the unit name of the serie values.
    //
    // Returns: The unit name of the serie values.
    function getUnitName()
    {
        return m_unitname;
    }

    // (METHOD) LokadSerie.setUnitName
    // Description: Sets the unit name of the data to a specified value.
    //
    // Syntax: setUnitName(unitname: String)
    //
    // Remarks: Sets the serie unit name the data to a specified |unitname|. This
    //      unit is provided for user convenience noly and has no impact on the
    //      time-serie semantic.
    //
    // Param(unitname): The |unitname| for which the unit name should be set.
    //      It must be less than 64 characters. Additionally, the following
    //      list of characters ; [ ] < > \ / " ' are prohibited.
    function setUnitName(unitname)
    {
        m_unitname = unitname.substr(0, 64);
    }

    // (METHOD) LokadSerie.getUtcOffset
    // Description: Gets the UTC offset of the serie times.
    //
    // Syntax: LokadSerie.getUtcOffset() : Double
    //
    // Remarks: Returns the UTC offset of the serie times.
    //
    // Returns: The UTC offset of the serie times, which expresses the number
    //      of hours of difference between the local time and the UTC time.
    function getUtcOffset()
    {
        return m_utcoffset;
    }

    // (METHOD) LokadSerie.setUtcOffset
    // Description: Sets the UTC offset of the data to a specified value.
    //
    // Syntax: setUtcOffset(offset: double)
    //
    // Remarks: Sets the UTC offset of the data to a specified |offset|. The
    //      UTC offset expresses the number of hours of difference between the
    //      local time and the UTC time. The simple equation local time = UTC
    //      time + offset expresses the relationship between local time and UTC
    //      time.
    //
    // Param(unitname): The |value| for which the unit name should be set.  It
    //      must be less than 64 characters.  Additionally, the following list
    //      of characters ; [ ] < > \ / " ' are prohibited.
    function setUtcOffset(offset)
    {
        m_utcoffset = offset;
    }

    // (METHOD) LokadSerie.getTimeValues
    // Description: Gets the time-serie values of this serie.
    //
    // Syntax: LokadSerie.getTimeValues() : Array(LokadTimeSerie)
    //
    // Remarks: Returns the time-serie values of this serie.
    //
    // Returns: The time-serie values of this serie.
    function getTimeValues()
    {
        return m_timevalues;
    }

    // (METHOD) LokadSerie.setTimeValues
    // Description: Sets the time-serie values for this serie.
    //
    // Syntax: setTimeValues(timevalues: Array(LokadTimeValues))
    //
    // Remarks: Sets the time-serie values for this serie.
    //
    // Param(timevalues): An Array of LokadTimeValues to assign to the serie.
    function setTimeValues(timevalues)
    {
        m_timevalues = timevalues;
    }

    // (METHOD) LokadSerie.addTimeValue
    // Description: Adds a time-serie value to this serie.
    //
    // Syntax: addTimeValue(timevalue: LokadTimeValues)
    //
    // Remarks: Adds a time-serie value to this serie.
    //
    // Param(timevalue): A |timevalue| to add to the time-serie Array.
    function addTimeValue(timevalue)
    {
        m_timevalues.push(timevalue);
    }

    var m_name, m_unitname, m_utcoffset, m_timevalues;
}




// (CLASS) LokadForecast
// Category: Lokad Forecaster
// Derives: LokadSerie
// Description: A class for storing a forecast from the server.
// Remarks: LokadForecast is the result of the Lokad server running a forecast
//      and returning the results.  The forecast data can be retrieved via
//      the LokadForecast.getTimeValues() method.
class LokadForecast extends LokadSerie
{

}




// (CLASS) LokadTimeValue
// Category: Lokad Forecaster
// Derives: Object
// Description: A class for storing a time-serie pair.
// Remarks: LokadTimeValue is used for storing a time-serie pair.  It has two
//      properties: time and value.
class LokadTimeValue
{
    var time, value;
}
