/*---------------------------------------------------------------------------

    Filename: lokad_task.js
    Project:  Lokad API
    (C) Copyright 2008, Kirix Corporation, All Rights Reserved.

    Summary:         A class for storing a forecast task based on a serie.
    Developer(s):    Jeff Engel

---------------------------------------------------------------------------*/


LokadTask.Sum               = 1;
LokadTask.Minimum           = 2;
LokadTask.Maximum           = 3;
LokadTask.First             = 4;
LokadTask.Last              = 5;
LokadTask.Average           = 6;
LokadTask.Count             = 7;
LokadTask.StandardDeviation = 8;
LokadTask.Variance          = 9;

LokadTask.QuarterHour       = 1;
LokadTask.HalfHour          = 2;
LokadTask.Hour              = 3;
LokadTask.Day               = 4;
LokadTask.Week              = 5;
LokadTask.Month             = 6;
LokadTask.Quarter           = 7;
LokadTask.Semester          = 8;
LokadTask.Year              = 9;



// (CLASS) LokadTask
// Category: Lokad Forecaster
// Derives: Object
// Description: A class for storing a forecast task based on a serie.
// Remarks: The LokadSerie class is equivalent to a forecast Task that is
//      stored on the Lokad servers.
//
// Property(LokadTask.Sum)               The sum of the values within the interval.
// Property(LokadTask.Minimum)           The lowest value within the interval.
// Property(LokadTask.Maximum)           The greatest value within the interval.
// Property(LokadTask.First)             The value of the earliest time-value within the interval.
// Property(LokadTask.Last)              The value of the latest time-value pair within the interval.
// Property(LokadTask.Average)           The average value of the values within the interval.
// Property(LokadTask.Count)             The number of values within the interval.
// Property(LokadTask.StandardDeviation) The standard deviation of the set of values within the interval.
// Property(LokadTask.Variance)          The variance of the set of values within the interval.
//
// Property(LokadTask.QuarterHour)       An interval of 15 minutes.
// Property(LokadTask.HalfHour)          An interval of 30 minutes.
// Property(LokadTask.Hour)              An interval of 1 hour.
// Property(LokadTask.Day)               An interval of 1 day.
// Property(LokadTask.Week)              An interval of 1 week.
// Property(LokadTask.Month)             An interval of 1 month.
// Property(LokadTask.Quarter)           An interval of 1 quarter (3 months).
// Property(LokadTask.Semester)          An interval of 1 semester (6 months).
// Property(LokadTask.Year)              An interval of 1 year.
class LokadTask
{
    // (CONSTRUCTOR) LokadTask.constructor
    // Description: Creates a new LokadTask object.
    //
    // Syntax: LokadSerie(name : String,
    //                    serie: LokadSerie)
    //
    // Remarks: Creates a new LokadTask object with the identifier |name| and
    //      assigned to the specified |serie|.
    //
    // Param(name): The unique name of the LokadTask as it will be stored and
    //      referenced on the Lokad servers. It should be distinct from any
    //      other time-serie names existing in your account and should be less
    //      than 64 characters. Additionally, the following list of characters
    //      ; [ ] < > \ / " ' are prohibited.
    // Param(serie): The |serie| that this LokadTask will be assigned to. Note
    //      that multiple forecasting tasks can be assigned to a single serie.
    function LokadTask(name, serie)
    {
        m_name = name.substr(0, 64);
        m_serie = serie;
        m_aggregator = 1;
        m_period = 4;
        m_periodstart = undefined;
        m_pastperiods = 0;
        m_futureperiods = 64;
    }


    // (METHOD) LokadTask.getName
    // Description: Gets the identifier of this task object.
    //
    // Syntax: LokadTask.getName() : String
    //
    // Remarks: Returns the identifier of this task object.
    //
    // Returns: The identifier of this task object as a string.
    function getName()
    {
        return m_name;
    }


    // (METHOD) LokadTask.getSerie
    // Description: Gets the serie this task object is assigned to.
    //
    // Syntax: LokadTask.getSerie() : LokadSerie
    //
    // Remarks: Returns the serie this task object is assigned to.
    //
    // Returns: The serie this task object is assigned to.
    function getSerie()
    {
        return m_serie;
    }


    // (METHOD) LokadTask.getAggregator
    // Description: Gets the forecast data aggregator.
    //
    // Syntax: LokadTask.getAggregator() : int
    //
    // Remarks: Returns the forecast data aggregator, which defines how values
    //      get combined into a time-interval.
    //
    // Returns: The forecast data aggregator.
    function getAggregator()
    {
        return m_aggregator;
    }


    // (METHOD) LokadSerie.setAggregator
    // Description: Sets the forecast data aggregator to a specified type.
    //
    // Syntax: setAggregator(aggregator: int)
    //
    // Remarks: Sets the forecast data aggregator a specified |aggregator|.
    //      This defines how values get combined into a time-interval. If the
    //      time-series are equispaced and the imput period is the same as the
    //      forecast period, then the aggregator has little importance.  The
    //      choice of aggregator depends of the meaning carried by the
    //      time-series. For example, Sum can be used to aggregate daily sales
    //      into weekly totals, whereas Average can be used to smooth the price
    //      of a stock price over a one-month period.
    //
    // Param(aggregator): The |aggregator| to use for the forecast data. Either
    //      LokadTask.Sum, LokadTask.Minimum, LokadTask.Maximum,
    //      LokadTask.First, LokadTask.Last, LokadTask.Average,
    //      LokadTask.Count, LokadTask.StandardDeviation or LokadTask.Variance.
    function setAggregator(aggregator)
    {
        m_aggregator = aggregator;
    }


    // (METHOD) LokadTask.getPeriod
    // Description: Gets the forecast period.
    //
    // Syntax: LokadTask.getPeriod() : int
    //
    // Remarks: Returns the forecast period.
    //
    // Returns: The forecast period.
    function getPeriod()
    {
        return m_period;
    }


    // (METHOD) LokadSerie.setPeriod
    // Description: Sets the forecast period to a specified interval.
    //
    // Syntax: setPeriod(interval: int)
    //
    // Remarks: Sets the forecast period to a specified |interval|. This
    //      specifies the time interval that should be used when computing the
    //      forecasts.
    //
    // Param(interval): The |interval| to use for the forecast data. Either
    //      LokadTask.QuarterHour, LokadTask.HalfHour, LokadTask.Hour,
    //      LokadTask.Day, LokadTask.Week, LokadTask.Month, LokadTask.Quarter,
    //      LokadTask.Semester or LokadTask.Year.
    function setPeriod(interval)
    {
        m_period = interval;
    }


    // (METHOD) LokadTask.getPeriodStart
    // Description: Gets the period start boundary.
    //
    // Syntax: LokadTask.getPeriodStart() : Date
    //
    // Remarks: Returns the period start boundary.
    //
    // Returns: The period start boundary.
    function getPeriodStart()
    {
        return m_periodstart;
    }


    // (METHOD) LokadSerie.setPeriodStart
    // Description: Sets the period start boundary.
    //
    // Syntax: setPeriodStart(startdate: Date)
    //
    // Remarks: Sets the period start inteval to a specified |startdate|. 
    //      For example, if you want your monthly forecasts to start the 17th
    //      of each month, just set January 17th 2007 as the period start value
    //      (year and month have no importance in this case). When this
    //      attribute is omitted, the period start is implicitly defined as
    //      January 1st 2001, the first day of the 21st century.
    //
    // Param(startdate): The |startdate| to use as the period start boundary.
    function setPeriodStart(startdate)
    {
        m_periodstart = startdate;
    }


    // (METHOD) LokadTask.getPastPeriods
    // Description: Gets the number of past periods to include in the
    //      time-series display.
    //
    // Syntax: LokadTask.getPastPeriods() : int
    //
    // Remarks: Returns the number of past periods to include in the
    //      time-series display.
    //
    // Returns: The number of past periods to include in the time-series
    //      display.
    function getPastPeriods()
    {
        return m_pastperiods;
    }


    // (METHOD) LokadSerie.setPastPeriods
    // Description: Sets the number of past periods to include in the
    //      time-series display.
    //
    // Syntax: setPastPeriods(count: int)
    //
    // Remarks: Sets the number of past periods to include in the
    //      time-series display. Note that this has no impact on the forecast
    //      values.
    //
    // Param(count): The number of past periods to use.
    function setPastPeriods(count)
    {
        m_pastperiods = count;
    }


    // (METHOD) LokadTask.getFuturePeriods
    // Description: Gets the number of future periods that will be forecasted.
    //
    // Syntax: LokadTask.getFuturePeriods() : int
    //
    // Remarks: Returns the number of future periods that will be forecasted.
    //
    // Returns: The number of future periods that will be forecasted.
    function getFuturePeriods()
    {
        return m_futureperiods;
    }


    // (METHOD) LokadSerie.setFuturePeriods
    // Description: Sets the number of future periods to include in the
    //      forecast.
    //
    // Syntax: setFuturePeriods(count: int)
    //
    // Remarks: Sets the number of future periods to include in the forecast.
    //      This indicates how many periods are to be forecast starting from
    //      the last time-value pair available. For example, if you want to
    //      retrieve weekly forecasts for the next 8 weeks, the |count|
    //      parameter should be set to 8. For performance reasons, this
    //      parameter is upper-bounded; the actual upper-bound depends on
    //      the period interval. For LokadSerie.Hour, LokadSerie.HalfHour and
    //      LokadSerie.QuarterHour, the limit is set to two weeks, e.g. 336,
    //      672 and 1344 respectively. For all all the other period intervals,
    //      the limit is set to 64.
    //
    //
    // Param(count): The number of future periods to use.
    function setFuturePeriods(count)
    {
        m_futureperiods = count;
    }



    // private member variables
    var m_name, m_serie, m_aggregator, m_period,
        m_periodstart, m_pastperiods, m_futureperiods;
}
