/*---------------------------------------------------------------------------

    Filename: main.js
    Project:  Lokad Forecaster
    (C) Copyright 2008, Kirix Corporation, All Rights Reserved.

    Summary:         Bootstrapper script to run the Lokad Forecaster extension
    Developer(s):    Jeff Engel

---------------------------------------------------------------------------*/


include "lokad_client.js";
include "options_form.js";
include "utils/logo_login_form.js";
include "utils/tools_menu_inserter.js";



// main entry point to the program; launch the program, either immediately if
// run as as script or else install to the tools menu if run as an extension
if (Extension.isContextPackage())
{
    // program is running as an extension, install to the "Tools" menu
    var menu_item = insertToolsMenuItem("Lokad Forecaster");

    // when triggered, display the main form
    menu_item.click.connect(initScript);
}
 else
{
    // program is running as a script, start immediately
    initScript();
}

function initScript()
{
    var launcher = new LokadForecaster;
    launcher.run();

    // avoid scope issues
    if (!initScript.launchers)
        initScript.launchers = [];
    initScript.launchers.push(launcher);
}

// start the main application loop
Application.run();





class LokadForecaster
{
    var m_db, m_prefs, m_optionsform, m_client, m_job;
    
    function LokadForecaster()
    {
        m_db = new DbConnection;
        m_prefs = new HostPreferences;
        m_client = new LokadClient('http://ws.lokad.com/TimeSeries.asmx');
    }

    function run()
    {
        m_optionsform = new OptionsForm();
        m_optionsform.forecastClick.connect(this, getAuthInfo);
        m_optionsform.setSourceTable(HostApp.getCurrentLocation());
        m_optionsform.m_client = m_client;
        m_optionsform.center();
        m_optionsform.show();
    }
    
    function getAuthInfo(sender)
    {
        var username = m_prefs.getValue("extensions.lokadforecaster.username");
        var password = SymmetricCrypt.decryptString(m_prefs.getValue(
                                        "extensions.lokadforecaster.password"));

        var loginForm = new LogoLoginForm();
        loginForm.ok_click.connect(this, processLogin);
        loginForm.cancel_click.connect(this, cancelLogin);
        loginForm.createaccount_click.connect(this, createLogin);
        loginForm.center();
        loginForm.m_optionsform = sender;
        processLogin(loginForm, [username, password, false]);
    }
    
    function processLogin(sender, args)
    {
        sender.show(false);
        var username = args[0];
        var password = args[1];
        
        if (args[2])
        {
            m_prefs.setValue("extensions.lokadforecaster.username", username);
            m_prefs.setValue("extensions.lokadforecaster.password",
                                       SymmetricCrypt.encryptString(password));
        }

        m_client.setAuthentication(username, password);

        // if the username and password are correct run the forecaster;
        // otherwise show the login dialog
        if (m_client.isAuthenticated())
        {
            m_client.setPartner(13247);
            runForecaster();
        }
        else
            sender.show(true);
    }
    
    function cancelLogin(sender)
    {
        sender.show(false);
    }
    
    function createLogin(sender)
    {
        HostApp.openWeb("http://app.lokad.com/CreateAccount.aspx");
    }
    
    
    function runForecaster()
    {

        // generate a unique number to use for identifying theses series
        var serieSetNumber = +m_prefs.getValue("extensions.lokadforecaster.lastserie");
        serieSetNumber++;
        m_prefs.setValue("extensions.lokadforecaster.lastserie", serieSetNumber);
        var time = new String(System.clock());
        serieSetNumber += "." + time.substr(time.length - 3);


        m_job = HostApp.createJob();
        var baseTitle = "Lokad Forecaster: " + m_optionsform.getSourceTable();
        m_job.setTitle(baseTitle + " - Uploading");
        m_job.setMaximum(100);
        m_job.start();

        try
        {

            // save the information about the source column

            var columnObj = undefined, columnDeclaration = "";
            var result = m_db.describeTable(m_optionsform.getSourceTable());
            if (!result)
                throw "Error looking up information about the source table.";
            for (var i = 0; i < result.length && !columnObj; i++)
                if (result[i].name == m_optionsform.getSeriesColumn())
                    columnObj = result[i];

            if(columnObj.type == DbType.Character)
                columnDeclaration = "Character(" + columnObj.width + ")";
            else if(columnObj.type == DbType.WideCharacter)
                columnDeclaration = "WideCharacter(" + columnObj.width + ")";
            else if(columnObj.type == DbType.Binary)
                columnDeclaration = "Binary";
            else if(columnObj.type == DbType.Numeric)
                columnDeclaration = "Numeric(" + columnObj.width + ","
                                  + columnObj.scale + ")";
            else if(columnObj.type == DbType.Double)
                columnDeclaration = "Double(" + columnObj.width + ","
                                  + columnObj.scale + ")";
            else if(columnObj.type == DbType.Integer)
                columnDeclaration = "Integer";
            else if(columnObj.type == DbType.Date)
                columnDeclaration = "Date";
            else if(columnObj.type == DbType.DateTime)
                columnDeclaration = "DateTime";
            else if(columnObj.type == DbType.Boolean)
                columnDeclaration = "Boolean";

            m_job.setValue(1);



            // create the LokadSerie object(s) and LokadTask object(s)

            var series = new Array, tasks = new Array, groupids = [], offset,
                lastserie = undefined, lastgroupid = undefined, lastidnum = 0;
            var multiple = m_optionsform.getSerieIdColumn() !== undefined;
            var today = new Date();
            var offset = -today.getTimezoneOffset() / 60;
            var result = m_db.execute(!multiple
                                ? "SELECT " + m_optionsform.getTimeColumn()
                                    + "," + m_optionsform.getSeriesColumn()
                                    + " FROM " + m_optionsform.getSourceTable()
                                : "SELECT " + m_optionsform.getTimeColumn()
                                    + "," + m_optionsform.getSeriesColumn()
                                    + "," + m_optionsform.getSerieIdColumn()
                                    + " FROM " + m_optionsform.getSourceTable()
                                    + " ORDER BY "
                                    + m_optionsform.getSerieIdColumn());
            if(!result || result.getColumnCount() != (multiple ? 3 : 2))
                throw "Error retrieving the data from the source table.";

            if(!multiple)
            {
                groupids.push("Series");
                lastserie = new LokadSerie("KirixStrata" + serieSetNumber);
                lastserie.setUtcOffset(offset);
                series.push(lastserie);

                var task = new LokadTask(lastserie.getName(), lastserie);
                task.setAggregator("Sum");
                task.setPeriod(m_optionsform.getInterval());
                task.setFuturePeriods(m_optionsform.getFutureRecords());
                tasks.push(task);
            }

            while(result.next() != null)
            {
                if(multiple && result[2] != lastgroupid)
                {
                    lastgroupid = result[2];
                    groupids.push(lastgroupid);
                    lastidnum++;
                    lastserie = new LokadSerie("KirixStrata" + serieSetNumber
                                         + "-" + lastidnum);
                    lastserie.setUtcOffset(offset);
                    series.push(lastserie);

                    var task = new LokadTask(lastserie.getName(), lastserie);
                    task.setAggregator("Sum");
                    task.setPeriod(m_optionsform.getInterval());
                    task.setFuturePeriods(m_optionsform.getFutureRecords());
                    tasks.push(task);

                }

                var ts = new LokadTimeValue;
                ts.time = new Date(result.getValue(0));
                ts.value = result.getValue(1);
                lastserie.addTimeValue(ts);
            }
            if(lastserie.getTimeValues().length > 25000)
                alert("Lokad Forecaster Warning: Only the first 25,000 records"
                    +" will be used.");

            m_job.setValue(25);
            if(m_job.isCancelling())
                throw new Error();



            // send the data to the server; series are sent individually to
            // provide better progress bar feedback and allow quicker cancelling

            for(var i in series)
            {
                m_client.addSerie(series[i]);

                m_job.setValue(25 + 70*i/tasks.length);
                if(m_job.isCancelling())
                    throw new Error();
            }

            m_job.setValue(95);



            // create the tasks on the server

            if(tasks.length == 1)
                m_client.addTask(tasks[0]);
            else
                m_client.addTasks(tasks);

            m_job.setValue(100);



            // pause to allow the forecasts to generate

            var max = m_optionsform.getDelay() * 60;
            m_job.setValue(0);
            m_job.setMaximum(0);
            for(var second = 0; second < max; second++)
            {
                m_job.setTitle(baseTitle + " - (Waiting "
                             + _zeroPad(Math.floor((max - second)/3600), 2) + ":"
                             + _zeroPad(Math.floor((max - second)/60) % 60, 2)
                             + ":" + _zeroPad((max - second) % 60, 2) + ")");
                System.sleep(1000);
                if(m_job.isCancelling())
                    throw new Error();
            }

            
            m_job.setValue(0);
            m_job.setTitle(baseTitle + " - (Downloading)");
            m_job.setMaximum(100);


            // download the forecasts from the server

            var forecasts = new Array;
            for(var i in tasks)
            {
                forecasts.push(m_client.getForecast(tasks[i]));

                m_job.setValue(90*i/tasks.length);
                if(m_job.isCancelling())
                    throw new Error();
            }

            m_job.setValue(90);



            // create the destination table

            if(m_db.execute("DROP TABLE IF EXISTS "
                            + m_optionsform.getDestinationTable()) == null)
                throw "Error saving the forecast data";
            var createSqlStatement = "CREATE TABLE "
                            + m_optionsform.getDestinationTable() + " (Time DATE"
                            + (m_optionsform.getInterval() <= 3 ? "TIME" : "");
            for(var i in groupids)
                createSqlStatement += ", " + _makeColumnName(groupids[i]) +
                                      " " + columnDeclaration;
            createSqlStatement += ")";
            var result = m_db.execute(createSqlStatement);
            if(result == null)
                throw "Error creating the forecast table.";

            m_job.setValue(92);
            if(m_job.isCancelling())
                throw new Error();



            // load the data into an array

            var numrecords = m_optionsform.getFutureRecords();
            var inserter = m_db.bulkInsert(m_optionsform.getDestinationTable());
            for(var recordnum = 0; recordnum < numrecords; recordnum++)
            {
                inserter[0] = forecasts[0].getTimeValues()[recordnum].time;
                for(var i = 0; i < forecasts.length; i++)
                    inserter[i + 1] =
                            forecasts[i].getTimeValues()[recordnum].value;
                inserter.insertRow();
            }
            inserter.finishInsert();



            // clean up

            m_job.setValue(100);
            m_job.setTitle(baseTitle);
            m_job.finish();

            HostApp.refresh();

            if (m_optionsform.getOpenOnceCompleted())
                HostApp.open(m_optionsform.getDestinationTable());
        }
        catch(e)
        {
            if(e.length)
                m_job.setTitle(baseTitle + " - " + e);
            else
                m_job.setTitle(baseTitle + " - (Unknown error)");

            m_job.cancel();

            if(m_optionsform.getOpenOnceCompleted() && e.length)
                alert("Lokad Forecaster:\n" + e);
        }
    }



    
    function _zeroPad(number, count)
    {
        var str = "00000000" + number;
        return str.substr(str.length - count);
    }
    
    function _makeColumnName(str)
    {
        return str.replace(new RegExp("[^A-Za-z0-9]"), "_");
    }
}
